# -*- coding: utf-8 -*-
import logging

from ast import literal_eval

from odoo.osv.expression import normalize_domain, is_operator
from openerp.addons.base.maintenance.migrations import util

_logger = logging.getLogger(__name__)

LEFT = 0
RIGHT = 1


class Tree:
    def __init__(self, domain):
        if not domain:
            self.root = None
            return

        self.root = Node(domain[0])
        current_node = self.root
        for d in domain[1:]:
            if is_operator(d):
                new_node = Node(d)
                current_node.append(new_node)
                current_node = new_node
            else:
                current_node = current_node.append(Node(d))

    def __str__(self):
        return str(self.root)

    def remove_node(self, field, current_node=False):
        if not self.root:
            return
        if not current_node:
            current_node = self.root
        # Make a copy of children since it can be changed during remove_node
        # And thus you'll miss some node
        for c in list(current_node.children):
            self.remove_node(field, c)

        if len(current_node.value) == 3 and field == current_node.value[0]:
            # We are the root, so the tree become empty
            if not current_node.parent:
                self.root = None
                return

            parent = current_node.parent
            parent.prune(current_node, self)

    def to_domain(self):
        if not self.root:
            return []
        return self.root.to_domain()


class Node:
    def __init__(self, value, parent=False, left=False, right=False):
        self.parent = parent
        self.value = value
        self.children = []

    def __str__(self, level=0):
        ret = "    "*level+repr(self.value)+"\n"
        for child in self.children:
            ret += child.__str__(level+1)
        return ret

    def to_domain(self):
        return [self.value] + sum([c.to_domain() for c in self.children], [])

    def append(self, node):
        if len(self.children) == 2 or (self.value == '!' and len(self.children) == 1):
            if not self.parent:
                raise ValueError("Domain malformed")
            return self.parent.append(node)

        self.children.append(node)
        node.parent = self
        return self

    def prune(self, child, tree):
        self.children.remove(child)

        if not self.children: # Operator ! that we should remove too
            if self.parent:
                self.parent.prune(self, tree)
            else:
                tree.root = None

        if self.children:
            if self.parent:
                idx = self.parent.children.index(self)
                self.parent.children[idx] = self.children[0]
                self.children[0].parent = self.parent
            else:
                tree.root = self.children[0]
                self.children[0].parent = False


def migrate(cr, version):
    # FIX SA: lead not assigned (TFR) button
    cr.execute("""
        UPDATE ir_act_server SET code = '
def get_eval_domain(domain):
  filter = env[''ir.filters''].new()
  filter[''name''] = "TEMP"
  filter[''domain''] = domain or ''[]''
  filter[''model_id''] = ''crm.lead''
  return filter._get_eval_domain()

#TFR, CDU, HBR, SBR
if env.user.id not in (104, 1265551, 892380, 1481):
  raise Warning("You are not authorized to launch this action")

create_date = datetime.datetime.today() - datetime.timedelta(days=30)
create_date = create_date.strftime("%Y-%m-%d %H:%M:%S")

unassigned_lead = env[''crm.lead''].search([
  (''team_id'', ''='', record.id),
  (''user_id'', ''='', False),
  (''create_date'', ''>'', create_date),
])
total_unassigned = len(unassigned_lead)
remaing_unassigned = env[''crm.lead''].browse(unassigned_lead.ids)
message = "Total unassigned: %s\\n" % total_unassigned
for user in env[''crm.team.member''].search([(''active'', ''='', True), (''crm_team_id'', ''='', record.id)]):
  domain = [''&'', (''id'', ''in'', unassigned_lead.ids)] + (get_eval_domain(user.assignment_domain) or [(1, ''='', 1)])
  assigned_ids = env[''crm.lead''].search(domain)
  remaing_unassigned -= assigned_ids
  message += "%s - %s: %s lead can be assigned\\n" % (user.user_id.login, user.assignment_domain, len(assigned_ids))
  message += "last modified by %s on %s\\n\\n" % (user.write_uid.login, user.write_date)

message += "\\n%s leads cannot be assigned\\n" % (len(remaing_unassigned))
message += "Sample: %s" % str(remaing_unassigned[:10].ids)
action = (message, remaing_unassigned.ids)'
        WHERE id = 3976;
    """)

    # Make sur CST team does not get any lead assigned
    cr.execute("""UPDATE crm_team
        SET assignment_optout = True
        WHERE id in (63, 106,121,132,100,137,139,138,64,82);
    """)

    # Fix Domain that contain score
    cr.execute("""UPDATE crm_team_member
        SET assignment_domain = '[["country_id","in",["Colombia","Puerto Rico","Peru","Argentina","Chile"]]]'
        WHERE assignment_domain ilike '%["Colombia","Puerto Rico","Peru","Argentina","Chile"]%' and assignment_domain ilike '%score%';
    """)
    cr.execute("""UPDATE crm_team_member
        SET assignment_domain = '[["country_id","in",["Colombia","Puerto Rico","Peru","Argentina","Chile","Peru"]]]'
        WHERE assignment_domain ilike '%["Colombia","Puerto Rico","Peru","Argentina","Chile","Peru"]%' and assignment_domain ilike '%score%';
    """)
    cr.execute("""UPDATE crm_team_member
        SET assignment_domain = '[["country_id","in",["Mexico"]]]'
        WHERE assignment_domain ilike '%["Mexico"]%' and assignment_domain ilike '%score%';
    """)
    cr.execute("""UPDATE crm_team_member
        SET assignment_domain = '[["lang_id", "not ilike", "french"],
            ["country_id", "not in", ["France", "Canada", "Morocco", "Algeria", "Congo", "Tunisia", "Spain"]],
            "|", ["referred", "ilike", "trial"], "&", ["referred", "ilike", "download"],
            ["country_id", "in", ["Netherlands", "France", "Belgium", "Switzerland", "Luxembourg", "United Kingdom"]]]'
        WHERE id = 114;
    """)

    cr.execute("""UPDATE ir_filters
        SET domain = '["&","&","&","&","&","&","&","&","&","&", "&", "&", "&",
            ["create_date",">","2019-05-15 00:00:00"],
            ["type","=","lead"],
            ["country_id","!=",False],
            ["tag_ids","not ilike","event"],
            ["tag_ids","not ilike","reveal"],
            ["tag_ids","not ilike","plan_to_sell"],
            ["tag_ids","not ilike","Referral"],
            ["tag_ids","not ilike","Spam"],
            ["tag_ids","not ilike","plan_to_test"],
            ["referred","not ilike","question"],
            ["user_id","=",False],
            ["partner_assigned_id","=",False],
            ["source_id", "not in", (47,48)],
            "|",
                "&",
                      ("create_date", "<", (context_today()-datetime.timedelta(days=1)).strftime("%Y-%m-%d")),["team_id","ilike"," AM"],
                "&",
                      ("create_date", "<", (context_today()-datetime.timedelta(days=10)).strftime("%Y-%m-%d")),["team_id","ilike"," Direct"]]'
        WHERE id = 23858;
    """)

    cr.execute("""UPDATE ir_filters
        SET domain = '["&", "&", ["team_id", "=", 140], ["user_id", "=", False], ["create_date", "<=", (context_today()-datetime.timedelta(days=2)).strftime("%%Y-%%m-%%d")]]'
        WHERE id in (33784, 34420);
    """)

    # FIX domain with score: generic for domain that can be eval with literal eval (no datetime, no context)
    # Remove the score and score_ids nodes
    cr.execute("""SELECT f.id, f.domain FROM ir_filters f
    JOIN res_users u ON f.user_id = u.id
    where
        u.active is TRUE AND
        f.model_id = 'crm.lead' AND
        (f.domain ilike '%score%' or f.domain ilike '%score_ids%') AND
        f.domain not ilike '%context_today%';""")
    for fid, domain in cr.fetchall():
        domain = domain.replace("uid", "'uid'")
        try:
            normalized_domain = normalize_domain(literal_eval(domain))
            t = Tree(normalized_domain)
            t.remove_node('score')
            t.remove_node('score_ids')
            correct_domain = str(t.to_domain()).replace("'uid'", "uid")
            print(correct_domain)
            cr.execute("UPDATE ir_filters SET domain = %s WHERE id = %s;", (correct_domain, fid))
        except:
            _logger.warning("Failed to correct domain with id %s", fid)
